// **** BEGIN LICENSE BLOCK ****
//   Copyright(c) 2009-2010 MATSUZAWA Jiro
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
//   02110-1301, USA.  */
// **** END LICENSE BLOCK ****

var rencon;
if (!rencon) {
    rencon = {};
} else if (typeof rencon != "object") {
    throw new Error("");
}

rencon.Config = function(
      separator
      )
{
    if (typeof separator != "string" || separator == "") {
        this._separator = "=";
    } else {
        this._separator = separator;
    }
    this._entry = {}
}

rencon.Config.prototype = {
///////////////////////////////////////////////////
// public
///////////////////////////////////////////////////
    read : function(
        fname
        )
    {
        var stream;
        stream = this._openInputStream(fname);
        this._readConfig(stream);
        stream.close();
    },


    write : function(
        fname
        )
    {
        var stream;
        stream = this._openOutputStream(fname);
        this._writeConfig(stream);
        stream.close();
    },


    getValue : function(
        key
        )
    {
        var val;

        if (typeof key != "string") {
            return "";
        }

        val = this._entry[key];
        if (val == undefined) {
            return "";
        }
        return val;
    },


    setValue : function(
        key,
        val
        )
    {
        if (typeof key != "string" ||
            typeof val != "string" ||
            key == "")
        {
            return;
        }
        this._entry[key] = val;
    },


///////////////////////////////////////////////////
// private
///////////////////////////////////////////////////
    _openInputStream : function(
        fname
        )
    {
        var file;
        var stream;

        file = this._openFile(fname);
        if (!file.exists() || !file.isReadable()) {
            throw new Error("ERROR output file open NG");
        }

        stream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                    .createInstance(Components.interfaces.nsIFileInputStream);
        if (!stream) {
            throw new Error("ERROR input file open NG. file name:" + fname);
        }
        stream.init(file, 0x01, 0444, 0);
        stream.QueryInterface(Components.interfaces.nsILineInputStream);
        return stream;
    },


    _openOutputStream : function(
        fname
        )
    {
        var file;
        var stream;

        file = this._openFile(fname);

        stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                    .createInstance(Components.interfaces.nsIFileOutputStream);
        if (!stream) {
            throw new Error("ERROR output file open NG. file name:" + fname);
        }
        stream.init(file, 0x02 | 0x08 | 0x10 | 0x20, 0664, 0);
        return stream;
    },


    _openFile : function(
        fname
        )
    {
        var wkFile;
        if (typeof fname != "string" || fname == "") {
            throw new Error("Config. file name(" + fname + ") is invalid");
        }

        wkFile =  Components.classes["@mozilla.org/file/local;1"]
                    .createInstance(Components.interfaces.nsILocalFile);
        if (!wkFile) {
            throw new Error("cannot open " + fname);
        }
        wkFile.initWithPath(fname);
        return wkFile;
    },


    _readConfig : function(
        stream
        )
    {
        var str = {};
        var ret;
        do {
            ret = stream.readLine(str);
            this._setEntry(str.value);
        } while (ret);
    },


    _writeConfig : function(
        stream
        )
    {
        var conf_str;
        for (var key in this._entry) {
            conf_str = this._format(key, this._entry[key]);
            stream.write(conf_str, conf_str.length);
        }
        stream.flush();
    },


    _setEntry : function(
        str
        )
    {
        var sep_idx;
        var key;
        var val;

        if (this._isComment(str)) {
            return;
        }
        sep_idx = str.indexOf(this._separator);
        if (sep_idx == -1) {
            return;
        }
        key = str.substring(0, sep_idx);
        if (key == "") {
            return;
        }
        val = str.substring(sep_idx+this._separator.length, str.length);
        this._entry[key] = val;
    },


    _isComment : function(
        str
        )
    {
        if (str.charAt(0) == "#") {
            return true;
        }
        return false;
    },


    _format : function(
        key,
        val
        )
    {
        return key + this._separator + val + "\n";
    },
};
