// **** BEGIN LICENSE BLOCK ****
//   Copyright(c) 2009-2010 MATSUZAWA Jiro
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
//   02110-1301, USA.  */
// **** END LICENSE BLOCK ****

var rencon;
if (!rencon) {
    rencon = {};
} else if (typeof rencon != "object") {
    throw new Error("cannot use rencon");
}

rencon.WebOfScienceParser = function(){}
rencon.PageParser.cloneParser(rencon.WebOfScienceParser);

rencon.WebOfScienceParser.prototype = {
    pickoutBibRecord : function()
    {
        var recordList;
        var record;
        var title, creator, journal, volume, pubDate, pages, url;
        var recordInfos;
        recordList = this._getRecordList();
        for (var i = 0; i < recordList.length; ++i) {
            recordInfos = this._getRecordInfo(recordList[i]);

            title = this._getTitle(recordInfos);
            creator = this._getCreator(recordInfos);
            journal = this._getJournal(recordInfos);
            pubDate = this._getPubDate(recordInfos);
            volume = this._getVolume(recordInfos);
            pages = this._getPages(recordInfos);
            url = this._getUrl(recordList[i], i+1);

            record = new rencon.ArticleRecord(title, creator, pubDate, url, journal, volume, pages);
            rencon.bibTable.addRecord(record);
        }
    },


    _getRecordList : function()
    {
        var recordList = null;
        recordList = window._content.document.getElementById("chunk_data").getElementsByTagName("tr");
        if (!recordList) {
            throw new Error("no records information");
        }

        return recordList;
    },


    _getTitle : function(infos)
    {
        var sKey = "Title: ";
        var eKey = "Author(s): ";
        var start = infos.indexOf(sKey);
        if (start == -1) {
            return "";
        }
        start += sKey.length;

        var end = infos.indexOf(eKey, start);
        if (end == -1) {
            return "";
        }
        return infos.substring(start, end);
    },


    _getCreator : function(infos)
    {
        var sKey = "Author(s): ";
        var eKey = "Source: ";
        var start = infos.indexOf(sKey);
        if (start == -1) {
            return "";
        }
        start += sKey.length;

        var end = infos.indexOf(eKey, start);
        if (end == -1) {
            return "";
        }
        return infos.substring(start, end);
    },


    _getJournal : function(infos)
    {
        return this._getItem(infos, "Source: ");
    },


    _getPubDate : function(infos)
    {
        return this._getItem(infos, "Published: ");
    },


    _getVolume : function(infos)
    {
        var volStr = "";
        var vol = this._getItem(infos, "Volume: ", " ");
        var issue = this._getItem(infos, "Issue: ", " ");
        if (vol) {
            volStr += vol;
        }
        if (issue) {
            volStr = volStr + "(" + issue + ")";
        }

        return volStr;
    },


    _getPages : function(infos)
    {
        return this._getItem(infos, "Pages: ");
    },


    _getUrl : function(info, num)
    {
        var id = "links_openurl_" + num;
        url = window._content.document.getElementById(id);
        if (url) {
            return "(unknown)";
        }
        return "";
    },


    _getRecordInfo : function(item)
    {
        var recordInfos;
        recordInfos = item.getElementsByClassName("summary_data")[0].textContent;
        if (!recordInfos) {
            throw new Error("no records information");
        }

        return recordInfos;
    },


    _getItem : function(infos, item, endKey)
    {
        var sKey = item;
        var eKey = String.fromCharCode(0x20, 0xA0, 0xA0);
//         if (arguments.length => 3) {
        if (endKey) {
            eKey = endKey;
        }
        var start = infos.indexOf(sKey, start);
        if (start == -1) {
            return "";
        }
        start += sKey.length;

        var end = infos.indexOf(eKey, start);
        if (end != -1) {
            return infos.substring(start, end);
        } 
        eKey = String.fromCharCode(0x20, 0x20, 0x0A);
        end = infos.indexOf(eKey, start);
        if (end != -1) {
            return infos.substring(start, end);
        } 

        return "";
    },
};
